<?php
/**
 * Main plugin class.
 */
class PNDL_WidgetSaverPlugin
{
	private $pluginFile;
	private $wpdb;
	private $updateMessage = NULL;

	private function __construct($pluginFile)
	{
		global $wpdb;

		$this->pluginFile = $pluginFile;
		$this->wpdb = $wpdb;
	}

	/**
	 * Creates a new instance and registers the appropriate actions.
	 * @param string $pluginFile the path to the plugin file.
	 */
	public static function create($pluginFile)
	{
		// instantiate the plugin
		$localPlugin = new PNDL_WidgetSaverPlugin($pluginFile);

		// register hook for displaying warning messages inline within the Plugins table.
		if (is_admin())
		{
			// note that we're using "/" instead of DIRECTORY_SEPARATOR - it would appear that wordpress has the plugin
			// name registered using "/" regardless of platform.
			$pluginName = basename(dirname($pluginFile)) . "/" . basename($pluginFile);
			add_action("after_plugin_row_$pluginName", array($localPlugin, 'after_plugin_row'));

			// hook for admin_init: need to handle widget actions before widgets are processed by WP. parse_request
			// is not called for admin pages, so we do this in admin_init instead.
			add_action('admin_init', array($localPlugin, 'admin_init'));
			add_action('switch_theme', array($localPlugin, 'switch_theme'));
			add_action('widgets_admin_page', array($localPlugin, 'widgets_admin_page'));
		}
	}

	public function switch_theme($newTheme)
	{
		$widgetArray = get_option('pndl_stored_widgets');
		if ($widgetArray && $widgetArray[$newTheme])
		{
			update_option('sidebars_widgets', $widgetArray[$newTheme]);
		}
	}

	public function admin_init()
	{
		$widgetArray = get_option('pndl_stored_widgets');
		if ($_POST['resetWidgets'])
		{
			update_option('sidebars_widgets', NULL);
			$this->updateMessage = 'Widgets have been reset.';
		}
		else if ($_POST['restoreWidgets'])
		{
			if ($widgetArray && $widgetArray[get_current_theme()])
			{
				update_option('sidebars_widgets', $widgetArray[get_current_theme()]);
				$this->updateMessage = sprintf('Widgets have been restored for theme %s.', get_current_theme());
			}
			else
			{
				$this->updateMessage = sprintf('No widgets have been previously saved for theme %s.', get_current_theme());
			}
		}
		else if ($_POST['saveWidgets'])
		{
			if (!$widgetArray)
			{
				$widgetArray = array();					
			}
			$widgetArray[get_current_theme()] = get_option('sidebars_widgets');
			update_option('pndl_stored_widgets', $widgetArray);
			$this->updateMessage = sprintf('Widgets have been saved for theme %s.', get_current_theme());
		}
	}

	public function widgets_admin_page()
	{
		$widgetArray = get_option('pndl_stored_widgets');

		if (!is_null($this->updateMessage))
		{
			printf('<div class="updated below-h2"><p>%s</p></div>', $this->updateMessage);
		}
		
		$savedWidgetsAvailable = $widgetArray && $widgetArray[get_current_theme()];
		echo '<div class="wrap">';
		echo '<h3>' . __('Widget Saver') . '</h3>';
		printf('<form method="post" action="%s">', $_SERVER["REQUEST_URI"]);
		printf('<input class="button" name="saveWidgets" type="submit" value="%s" />', __("Save Widgets"));
		printf('<input class="button" name="restoreWidgets" type="submit" value="%s"%s />', __("Restore Widgets"),
			($savedWidgetsAvailable ? "" : " disabled"));
		printf('<input class="button" name="resetWidgets" type="submit" value="%s" />', __("Reset Widgets"));
		print('<form>');
		echo "</div>";
		
	}

	/**
	 * Callback implementation for adding messages after the plugin row in the plugins page.
	 */
	public function after_plugin_row()
	{
		// no messages to add at this stage.
	}
}
?>